/*
 * (c) 1999, David Leonard. <david.leonard@csee.uq.edu.au>
 * $Id: fd.c,v 1.1.1.1 1999/10/16 07:19:54 leonard Exp $
 */
/*
 * Unix file descriptor layer.
 */
#include <stdio.h>
#include <err.h>
#include <unistd.h>
#include <stdlib.h>

#include "layer.h"

static void	fd_send(struct layer *, void *, size_t);
static void	fd_delete(struct layer *);
static void	fd_selset(struct layer *, int *, fd_set *);
static void	fd_selrdy(struct layer *, fd_set *);

static struct layerclass fd_class = {
	"fd",
	above_recv,
	fd_send,
	fd_delete,
	fd_selset,
	fd_selrdy
};

struct fdlayer {
	struct layer l;
	int fd;
};

struct layer *
fd_newlayer(fd)
	int fd;
{
	struct fdlayer *fdl;

	fdl = (struct fdlayer *)malloc(sizeof *fdl);
	fdl->l.class = &fd_class;
	fdl->l.above = NULL;

	fdl->fd = fd;

	return (struct layer *)fdl;
}

static void
fd_send(l, buf, len)
	struct layer *l;
	void *buf;
	size_t len;
{
	struct fdlayer *fdl = (struct fdlayer *)l;

#ifdef DEBUG
	fprintf(stderr, "fd write(fd=%d,,len=%d)\n", fdl->fd, len);
#endif
	write(fdl->fd, buf, len);
}


static void
fd_delete(l)
	struct layer *l;
{
	struct fdlayer *fdl = (struct fdlayer *)l;

	close(fdl->fd);
	free(fdl);
}

static void
fd_selset(l, n, r)
	struct layer *l;
	int *n;
	fd_set *r;
{
	struct fdlayer *fdl = (struct fdlayer *)l;

	selset_add(n, r, fdl->fd);
}

static void
fd_selrdy(l, r)
	struct layer *l;
	fd_set *r;
{
	struct fdlayer *fdl = (struct fdlayer *)l;
	char buf[8192];
	int len;

	if (FD_ISSET(fdl->fd, r)) {
		len = read(fdl->fd, buf, sizeof buf);
#ifdef DEBUG
		fprintf(stderr, "fd: read(%d) -> %d\n", fdl->fd, len);
#endif
		if (len == -1)
			warn("read");
		else
			layer_recv(l, buf, len);
	}
}
