/*
 * (c) 1999, David Leonard. <david.leonard@csee.uq.edu.au>
 * $Id: layer.c,v 1.1.1.1 1999/10/16 07:19:54 leonard Exp $
 */
/*
 * Layer operations.
 */
#include <unistd.h>
#include <stdio.h>
#include <err.h>
#include <sys/types.h>
#include "layer.h"

void
no_recv(l, buf, size)
	struct layer *l;
	void * buf;
	size_t size;
{
}

void
no_send(l, buf, size)
	struct layer *l;
	void * buf;
	size_t size;
{
}

void
no_delete(l)
	struct layer *l;
{
}


void
no_selset(l, n, r)
	struct layer *l;
	int *n;
	fd_set *r;
{
}

void
no_selrdy(l, r)
	struct layer *l;
	fd_set *r;
{
}

/* Perform one selset/select/selrdy. */
void
layer_select(l)
	struct layer *l;
{
	fd_set r;
	int n, ret;

	n = 0;
	FD_ZERO(&r);

	/* Pull out the fd's needed */
	layer_selset(l, &n, &r);
#ifdef DEBUG
	fprintf(stderr, "layer_select: about to select; n = %d\n", n);
#endif DEBUG

	/* Wait for one of the fds to become ready for read. */
	ret = select(n, &r, NULL, NULL, NULL);
	if (ret == -1)
		err(1, "select");

#ifdef DEBUG
	fprintf(stderr, "layer_select: %d fds ready\n", ret);
	for (--n; n > 0; --n)
		if (FD_ISSET(n, &r))
			fprintf(stderr, "layer_select: fd %d ready\n", n);
#endif

	/* Handle read-ready fds. */
	layer_selrdy(l, &r);
}

void
above_recv(l, buf, len)
	struct layer *l;
	void *buf;
	size_t len;
{
	layer_recv(l->above, buf, len);
}

void
below_send(l, buf, len)
	struct layer *l;
	void *buf;
	size_t len;
{
	if (l->below)
		layer_send(l->below, buf, len);
}

void
below_selset(l, n, r)
	struct layer *l;
	int *n;
	fd_set *r;
{
	if (l->below)
		layer_selset(l->below, n, r);
}

void
below_selrdy(l, r)
	struct layer *l;
	fd_set *r;
{
	if (l->below)
		layer_selrdy(l->below, r);
}

void
below_delete(l)
	struct layer *l;
{
	if (l->below)
		layer_delete(l->below);
}

#ifdef DEBUG
/* Debugging wrappers to help track flow of control. */
void
layer_recv(l, b, s)
	struct layer *l;
	void *b;
	size_t s;
{
	fprintf(stderr, "%s recv\n", l->class->name);
	(*l->class->recv)(l, b, s);
}

void
layer_send(l, b, s)
	struct layer *l;
	void *b;
	size_t s;
{
	fprintf(stderr, "%s send\n", l->class->name);
	(*l->class->send)(l, b, s);
}

void
layer_delete(l)
	struct layer *l;
{
	fprintf(stderr, "%s delete\n", l->class->name);
	(*l->class->delete)(l);
}

void
layer_selset(l, n, r)
	struct layer *l;
	int *n;
	fd_set *r;
{
	fprintf(stderr, "%s selset\n", l->class->name);
	(*l->class->selset)(l, n, r);
}

void
layer_selrdy(l, r)
	struct layer *l;
	fd_set *r;
{
	fprintf(stderr, "%s selrdy\n", l->class->name);
	(*l->class->selrdy)(l, r);
}
#endif
