/*
 * (c) 1999, David Leonard. <david.leonard@csee.uq.edu.au>
 * $Id: main.c,v 1.1.1.1 1999/10/16 07:19:54 leonard Exp $
 */

/*
 * Gateway between a pty and telnet for use with demand-ppp tunnels.
 *
 * This program is structured from 4 different layers.
 * The topmost layer, gw, acts as a gateway between its
 * two child layers. The pty layer maintains a continuously
 * open master pty, and the telnet layer takes care of
 * decoding/encoding the TELNET protocol. The lowest fd
 * layers each provide efficient access to the unix file
 * descriptor interface.
 *
 *	+--------------------------+
 *      |            gw            |
 *	+----------+----+----------+
 *	|   pty    |    |  telnet  |
 *	+----------+    +----------+
 *	|   fd     |    |   fd     |
 *	+----------+    +----------+
 *
 * The easiest way to test this program is as follows:
 *	$ ptymodem ptypf &
 *	$ sudo cu -x abnormal -l /dev/ttypf
 *      prompt> telnet localhost
 */

#include <stdio.h>
#include "layer.h"
#include "gw.h"

int
main(argc, argv)
	int argc;
	char *argv[];
{
	struct layer *gw;

	if (argc != 2) {
		fprintf(stderr, "usage: %s pty\n", argv[0]);
		exit(1);
	}

	/* Create top layer, which recursively creates lower layers. */
	gw = gw_newlayer(argv[1], "ptymodem> ");

	/* Main loop. */
	for (;;)
		layer_select(gw);
}
