/*
 * (c) 1999, David Leonard. <david.leonard@csee.uq.edu.au>
 * $Id: pty.c,v 1.1.1.1 1999/10/16 07:19:54 leonard Exp $
 */
/*
 * Pseudo-tty layer.
 */
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <paths.h>
#include <stdlib.h>
#include <errno.h>
#include <err.h>
#include <sys/param.h>

#include "layer.h"
#include "pty.h"
#include "fd.h"

static void pty_delete(struct layer *);

static struct layerclass pty_class = {
	"pty",
	above_recv,
	below_send,
	pty_delete,
	below_selset,
	below_selrdy
};

struct ptylayer {
	struct layer l;
	char path[MAXPATHLEN];
};

void
pty_reopen(l)
	struct layer *l;
{
	struct ptylayer *p = (struct ptylayer *)l;
	int fd;
	struct layer *fdl;

	if (l->below) {
#ifdef DEBUG
		fprintf(stderr, "pty_reopen: closing ...");
#endif
		layer_delete(l->below);
		sleep(1);
#ifdef DEBUG
		fprintf(stderr, "done\n");
#endif
	}
	l->below = NULL;

#ifdef DEBUG
	fprintf(stderr, "pty_reopen: open %s ...", p->path);
#endif
	revoke(p->path);
	fd = open(p->path, O_RDWR);
#ifdef DEBUG
	fprintf(stderr, " ret = %d\n", fd);
#endif

	if (fd == -1)
		return;

	fdl = fd_newlayer(fd);
	if (fdl == NULL)
		close(fd);
	else
		new_below(l, fdl);
}

struct layer *
pty_newlayer(path)
	const char *path;
{
	struct ptylayer *p;

	p = (struct ptylayer *)malloc(sizeof *p);
	p->l.class = &pty_class;
	p->l.above = NULL;
	p->l.below = NULL;

        sprintf(p->path, "%s", path);
        if (access(p->path, R_OK|W_OK) == -1) {
                if (errno != ENOENT)
                        err(1, "%s", p->path);
                sprintf(p->path, "%s%s", _PATH_DEV, path);
                if (access(p->path, R_OK|W_OK) == -1) {
                        if (errno != ENOENT)
                                err(1, "%s", p->path);
                        sprintf(p->path, "%s%s%s", _PATH_DEV, "pty", path);
                        if (access(p->path, R_OK|W_OK) == -1) {
                                if (errno != ENOENT)
                                        err(1, "%s", p->path);
				else
					err(1, "%s", path);
                        }
                }
        }

	pty_reopen((struct layer *)p);
	return (struct layer *)p;
}

static void
pty_delete(l)
	struct layer *l;
{
	struct ptylayer *p = (struct ptylayer *)l;

	below_delete(l);
	free(p);
}
